package main

import (
	"consumerDemo-json_go/sasl_sarama"
	"context"
	"flag"
	"fmt"
	"github.com/Shopify/sarama"
	"go.mongodb.org/mongo-driver/bson"
	"strings"
)

var (
	partitionConsumers = make(map[int32]*partitionMsgConsumer)
	brokerList         = flag.String("brokers", "", "REQUIRED: The comma separated list of brokers in the Kafka cluster")
	topic              = flag.String("topic", "", "REQUIRED: the topic to consume")
	group              = flag.String("group", "", "REQUIRED: the consumer group")
	user               = flag.String("user", "", "the user name")
	password           = flag.String("password", "", "the password")
	isPrintCkp         = flag.Bool("isPrintCkp", true, "print checkpoint or not")
)

func main() {
	flag.Parse()
	config := sarama.NewConfig()
	config.Version = sarama.V0_11_0_0
	config.Consumer.Return.Errors = true
	config.Consumer.Offsets.Initial = sarama.OffsetOldest
	config.Consumer.Fetch.Default = 20 << 20
	if *user != "" {
		config.Net.SASL.Enable = true
		config.Net.SASL.User = *user
		config.Net.SASL.Password = *password
		config.Net.SASL.Handshake = true
		config.Net.SASL.SCRAMClientGeneratorFunc = func() sarama.SCRAMClient { return &sasl_sarama.XDGSCRAMClient{HashGeneratorFcn: sasl_sarama.SHA512} }
		config.Net.SASL.Mechanism = sarama.SASLTypeSCRAMSHA512
	}

	// Start with a client
	client, err := sarama.NewClient(strings.Split(*brokerList, ","), config)
	if err != nil {
		panic(err)
	}
	defer func() { _ = client.Close() }()

	// Start a new consumer group
	group, err := sarama.NewConsumerGroupFromClient(*group, client)
	if err != nil {
		panic(err)
	}
	defer func() { _ = group.Close() }()

	// Track errors
	go func() {
		for err := range group.Errors() {
			fmt.Println("ERROR", err)
		}
	}()

	// Iterate over consumer sessions.
	ctx := context.Background()
	for {
		handler := demoConsumerGroupHandler{}
		// 开始消费
		err := group.Consume(ctx, []string{*topic}, handler)
		if err != nil {
			panic(err)
		}
	}
}

type demoConsumerGroupHandler struct {
}

func (demoConsumerGroupHandler) Setup(sess sarama.ConsumerGroupSession) error {
	setup := make(map[int32]bool)
	for _, part := range sess.Claims()[*topic] {
		if _, ok := partitionConsumers[part]; !ok {
			partitionConsumers[part] = &partitionMsgConsumer{}
		}

		setup[part] = true
	}

	var invalid []int32
	for p := range partitionConsumers {
		if _, ok := setup[p]; !ok {
			invalid = append(invalid, p)
		}
	}

	for _, p := range invalid {
		delete(partitionConsumers, p)
	}

	return nil
}

const CheckPoint = "Checkpoint"

func (demoConsumerGroupHandler) Cleanup(_ sarama.ConsumerGroupSession) error { return nil }

func (h demoConsumerGroupHandler) ConsumeClaim(sess sarama.ConsumerGroupSession, claim sarama.ConsumerGroupClaim) error {
	for msg := range claim.Messages() {
		if err := partitionConsumers[msg.Partition].consume(sess, msg); nil != err {
			return err
		}
	}
	return nil
}

type partitionMsgConsumer struct {
	lastPartitionSeq uint64
}

func (pmc *partitionMsgConsumer) consume(sess sarama.ConsumerGroupSession, msg *sarama.ConsumerMessage) error {
	var isTrue = true
	for _, header := range msg.Headers {
		if string(header.Key) == CheckPoint {
			sess.MarkMessage(msg, "")
			if !*isPrintCkp {
				isTrue = false
			}
		}
	}
	if isTrue {
		for _, header := range msg.Headers {
			fmt.Println(fmt.Sprintf("key: %s, value: %s", string(header.Key), string(header.Value)))
		}
		var raw bson.Raw
		raw = msg.Value
		cs := raw.String()

		fmt.Printf("-->[partition: %v, offset: %v]\n %+v\n", msg.Partition, msg.Offset, string(cs))
	}

	return nil
}
